"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecuteTranscriptionsUpload = exports.EnqueueTranscriptionsUpload = void 0;
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
const mssql_service_1 = require("../db/mssql.service");
const background_shared_1 = require("../shared/background.shared");
const transcriptions_upload_implementation_1 = require("../implementation/transcriptions-upload.implementation");
const supervisor_implementation_1 = require("../implementation/supervisor.implementation");
const promises_shared_1 = require("../shared/promises.shared");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const task_config_1 = require("../config/task.config");
let logger = log4js.getLogger("Background");
function EnqueueTranscriptionsUpload(queue, childTaskId) {
    let logLabel = `[activity.background::EnqueueTranscriptionsUpload] `;
    let rate;
    let items = 0;
    let hasItems = false;
    let childTask = {};
    let requiresReload = false;
    let executeQueue = 'TranscriptionsUpload';
    return background_shared_1.StartProcess(queue, true, false, false)
        .then(_ => {
        return redis_service_1.Redis.GetLengthFromList(executeQueue);
    })
        .then(result => {
        logger.info(`${logLabel}found on redis ${result}`);
        items = result;
        return Promise.resolve(false);
    })
        .then(result => {
        requiresReload = result || requiresReload;
        if (!items || (items && items <= 2)) {
            return redis_service_1.Redis.GetLengthFromList(`${executeQueue}Execute`);
        }
        return Promise.resolve(null);
    })
        .then(result => {
        if (result) {
            hasItems = true;
            let supervisor = new supervisor_implementation_1.SupervisorImplementation();
            return supervisor.RestoreQueue(executeQueue, 'Background');
        }
        return Promise.resolve(null);
    })
        .then(_ => {
        if (childTask && (hasItems || items)) {
            logger.info(`${logLabel}launching task`);
            let taskBackgroundProcess = {
                id: childTask.id || executeQueue,
                pid: process.pid,
                processor: childTask.processor,
                queue: executeQueue,
                rate: childTask.rate || 10,
                isOver: false,
                requiresReload: requiresReload
            };
            process.send(taskBackgroundProcess);
        }
        return SleepPromise(60);
    })
        .then(_ => {
        return EnqueueTranscriptionsUpload(queue, childTaskId);
    })
        .catch(err => {
        let errMsg = typeof err === 'string' ? err : (err.message || err.description);
        logger.error(`${logLabel}getting err: ${errMsg}`);
        return Promise.reject(err);
    });
}
exports.EnqueueTranscriptionsUpload = EnqueueTranscriptionsUpload;
function ExecuteTranscriptionsUpload(queue, executionCounter) {
    let logLabel = `[activity.background::ExecuteTranscriptionsUpload] `;
    executionCounter ?? (executionCounter = 0);
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (task_config_1.TRANSCRIPTION_ENGINE === 'DEEPGRAM') {
            let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.GetTranscriptionEngineAvailability();
        }
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_TRANSCRIPTION_ENGINE_NOT_AVAILABLE';
        return background_shared_1.StartProcess(queue, false, true, true);
    })
        .then(_ => {
        let transcriptionsUploadImpl = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
        return transcriptionsUploadImpl.BackgroundExecute(executionCounter);
    })
        .then(_ => {
        return ExecuteTranscriptionsUpload(queue, executionCounter);
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error(`${logLabel}getting err: ${errorMessage}`);
        console.error(err);
        return SleepPromise(30)
            .then(_ => Promise.resolve(false));
    });
}
exports.ExecuteTranscriptionsUpload = ExecuteTranscriptionsUpload;
function GetTask(taskId) {
    let cacheUpdateNeeded = false;
    let referenceTask;
    let referenceRedisTask;
    return redis_service_1.Redis.GetTask(taskId)
        .then((resultTask) => {
        referenceRedisTask = resultTask;
        if (referenceRedisTask) {
            return Promise.resolve(resultTask);
        }
        else {
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetTask(taskId);
        }
    }).then((resultTask) => {
        referenceTask = resultTask;
        if (referenceTask && !referenceRedisTask && cacheUpdateNeeded) {
            return redis_service_1.Redis.SaveTask(taskId, referenceTask);
        }
        else {
            return Promise.resolve(null);
        }
    }).then((result) => {
        if (referenceTask)
            return Promise.resolve(referenceTask);
        return Promise.resolve(null);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        return Promise.resolve(null);
    });
}
function SleepPromise(secondTime) {
    return new Promise(function (resolve, reject) {
        setTimeout(resolve, (1000 * secondTime));
    });
}
function ThrowPromise(secondTime) {
    return new Promise(function (resolve, reject) {
        setTimeout(() => {
            reject(new Error('WORKFLOW_TIMEOUT'));
        }, (1000 * secondTime));
    });
}
