"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecuteTask = exports.StartService = void 0;
const log4js = require("log4js");
const express = require("express");
const mssql_config_1 = require("../config/mssql.config");
const redis_init_1 = require("../db/redis.init");
const redis_service_1 = require("../db/redis.service");
const mssql_init_1 = require("../db/mssql.init");
const background_service_1 = require("./background.service");
const background_config_1 = require("../config/background.config");
const background_shared_1 = require("../shared/background.shared");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger("Background");
let app;
let sqlConnection;
let redisConnection;
function StartService() {
    let newConfig = JSON.parse(JSON.stringify(mssql_config_1.MSSQL_CONFIG));
    newConfig.pool.max = 10;
    newConfig.pool.idleTimeoutMillis = 5 * 1000;
    return Promise.resolve(true)
        .then(_ => {
        app = app ? app : express();
        return redisConnection ? null : redis_init_1.RedisInit(app);
    }).then(result => {
        redisConnection = result;
        return sqlConnection ? null : mssql_init_1.MsSqlInit(app, newConfig);
    }).then(results => {
        sqlConnection = results;
        return Promise.resolve(true);
    }).catch(error => {
        setTimeout(function () {
            process.exit();
        }, 1000);
    });
}
exports.StartService = StartService;
function ExecuteTask(taskId, queue, pid) {
    let newConfig = JSON.parse(JSON.stringify(mssql_config_1.MSSQL_CONFIG));
    newConfig.pool.max = 10;
    newConfig.pool.idleTimeoutMillis = 5 * 1000;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return VerifyService();
    })
        .then(_ => {
        let arrayPromise = [];
        if (background_config_1.DISABLED_TASK.includes(taskId)) {
            logger.info('ExecuteTask: Tarea desactivada:', taskId, process.pid);
            return Promise.all(arrayPromise);
        }
        switch (taskId) {
            case 'supervisor':
                arrayPromise.push(Supervisor(taskId, background_config_1.RELOAD_TIME));
                break;
            case 'TranscriptionsUploadEnqueue':
                arrayPromise.push(background_service_1.Background.EnqueueTranscriptionsUpload(taskId, background_config_1.SERVER_NAME));
                break;
            case 'TranscriptionsUpload':
                arrayPromise.push(background_service_1.Background.ExecuteTranscriptionsUpload(taskId));
                break;
        }
        return Promise.all(arrayPromise);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(error => {
        setTimeout(function () {
            process.exit();
        }, 1000);
    });
}
exports.ExecuteTask = ExecuteTask;
function VerifyService() {
    return Promise.resolve(true)
        .then(_ => {
        if (app && sqlConnection && redisConnection)
            return true;
        return SleepPromise(1);
    })
        .then(_ => {
        if (app && sqlConnection && redisConnection)
            return true;
        return VerifyService();
    })
        .then(_ => {
        return Promise.resolve(true);
    });
}
function SleepPromise(secondTime) {
    return new Promise(function (resolve, reject) {
        setTimeout(resolve, (1000 * secondTime));
    });
}
function Supervisor(taskId, scheduledReload) {
    let now = null;
    logger.info('[dispatcher.background::Supervisor]', taskId, process.pid, scheduledReload);
    console.log(taskId, process.pid, scheduledReload, new Date().toString());
    return background_shared_1.StartProcess(taskId, true, false, true)
        .then(_ => {
        let date = new Date();
        let hour = date.getHours() < 10 ? '0' + date.getHours().toString() : date.getHours().toString();
        let min = date.getMinutes() < 10 ? '0' + date.getMinutes().toString() : date.getMinutes().toString();
        now = hour + ':' + min;
        if (scheduledReload && scheduledReload === now) {
            logger.info(taskId, process.pid, 'parar tarea', scheduledReload);
            return redis_service_1.Redis.SaveTaskBackgroundStop('all', true);
        }
        return Promise.resolve(null);
    })
        .then(_ => {
        return SleepPromise(scheduledReload && scheduledReload === now ? background_config_1.MAXIMUM_TIME_TO_WAIT_TO_STOP_A_FAST_TASK_IN_SECONDS : 60);
    })
        .then(_ => {
        if (scheduledReload && scheduledReload === now) {
            return redis_service_1.Redis.DeleteTaskBackgroundStop();
        }
        return Promise.resolve(null);
    })
        .then(_ => {
        if (scheduledReload && scheduledReload === now) {
            logger.info(taskId, process.pid, 'reinicia tarea', scheduledReload);
            let taskBackgroundProcess = {
                id: taskId,
                pid: process.pid,
                processor: null,
                queue: null,
                rate: null,
                requiresReload: true
            };
            process.send(taskBackgroundProcess);
        }
        return Promise.resolve(null);
    })
        .then(_ => {
        return Supervisor(taskId, scheduledReload);
    })
        .catch(err => {
        let errMsg = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('Supervisor', errMsg);
        if (errMsg.startsWith('RELOAD')) {
            let taskBackgroundProcess = { id: taskId, pid: process.pid, processor: null, queue: null, rate: null, requiresStopping: true };
            process.send(taskBackgroundProcess);
            return SleepPromise(background_config_1.MAXIMUM_TIME_TO_WAIT_TO_STOP_A_FAST_TASK_IN_SECONDS);
        }
        return Supervisor(taskId, scheduledReload);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch(err => {
        let errMsg = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('Supervisor', errMsg);
        return Promise.reject(err);
    });
}
