"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LOGGER_CONFIG = exports.TASK_LOG_FILE = exports.LOG_LEVEL = exports.ENABLE_LOG_PM2_INTERCOM = void 0;
const path = require("path");
const stackTrace = require("stack-trace");
exports.ENABLE_LOG_PM2_INTERCOM = true;
exports.LOG_LEVEL = 'DEBUG';
exports.TASK_LOG_FILE = path.resolve(__dirname, '..', 'logs', 'Task.log');
let getLogPid = function () {
    let pid = process.pid.toString();
    return (pid + '     ').substring(0, 5);
};
let getLogLevel = function (loggingEvent) {
    let resultLength = 5;
    var level = loggingEvent.level.toString();
    return '[' + (level + Array(resultLength + 1).join(' ')).substring(0, resultLength) + ']';
};
let getLogCategory = function (loggingEvent) {
    let resultLength = 20;
    var category = loggingEvent.categoryName.toString() + '.log';
    return '[' + (category + Array(resultLength + 1).join(' ')).substring(0, resultLength) + ']';
};
let getLogContext = function (loggingEvent) {
    let resultLength = 30;
    let filePath = loggingEvent.fileName;
    let lineNumber = (loggingEvent.lineNumber || 0).toString();
    if (!filePath) {
        let trace = stackTrace.get();
        if (trace.length <= 15)
            return '[Unknown context               ]';
        let caller = trace[15];
        filePath = caller.getFileName();
        lineNumber = caller.getLineNumber().toString() || '';
        if (!filePath)
            return '[Unknown context               ]';
    }
    let fileName = path.basename(filePath) || '';
    let context = fileName + ':' + lineNumber;
    context = context.length > resultLength ? '...' + context.slice(-1 * (resultLength - ('...').length)) : context;
    return '[' + (context + Array(resultLength + 1).join(' ')).substring(0, resultLength) + ']';
};
let GetFileAppender = (logName, showContext, showCategory, maxFiles) => ({
    type: 'file',
    filename: path.resolve(__dirname, '..', 'logs', logName + '.log'),
    maxLogSize: 20 * 1000 * 1024,
    backups: maxFiles || 5,
    category: logName,
    layout: {
        type: 'pattern',
        pattern: (showCategory ? '%x{category} ' : '') + '%x{level} %d{yyyy-MM-dd hh:mm:ss.SSS} %x{pid} ' + (showContext ? '%x{context} ' : '') + '%m',
        tokens: {
            pid: getLogPid,
            level: getLogLevel,
            category: getLogCategory,
            context: getLogContext
        }
    }
});
let GetConsoleAppender = (logName, showContext, showCategory) => ({
    type: 'console',
    category: logName,
    layout: {
        type: 'pattern',
        pattern: (showCategory ? '%x{category} ' : '') + '%x{level} %d{yyyy-MM-dd hh:mm:ss.SSS} %x{pid} ' + (showContext ? '%x{context} ' : '') + '%m',
        tokens: {
            pid: getLogPid,
            level: getLogLevel,
            category: getLogCategory,
            context: getLogContext
        }
    }
});
let GetLogLevelAppender = (level, appender) => ({
    type: 'logLevelFilter',
    level: level,
    appender: appender,
});
let GetCategory = (appender, level, sendToErrorLog) => ({
    appenders: [
        appender,
        ...(sendToErrorLog ? ['ErrorFilter'] : []),
    ],
    level: level,
    enableCallStack: true,
});
exports.LOGGER_CONFIG = {
    appenders: {
        'Console': GetConsoleAppender('Console', true, false),
        'AccessLog': GetFileAppender('AccessLog', false, false),
        'Background': GetFileAppender('Background', true, false),
        'Cleaner': GetFileAppender('Cleaner', true, false),
        'Error': GetFileAppender('Error', true, true),
        'ErrorFilter': GetLogLevelAppender('ERROR', 'Error'),
        'IO': GetFileAppender('IO', true, false),
        'ServerScripts': GetFileAppender('ServerScripts', true, false),
        'Sql': GetFileAppender('Sql', true, false),
        'PublicScripts': GetFileAppender('PublicScripts', true, false),
        'Redis': GetFileAppender('Redis', true, false),
        'Task': GetFileAppender('Task', true, false),
    },
    categories: {
        'default': GetCategory('ServerScripts', exports.LOG_LEVEL, true),
        'Console': GetCategory('Console', exports.LOG_LEVEL, true),
        'AccessLog': GetCategory('AccessLog', exports.LOG_LEVEL, false),
        'Background': GetCategory('Background', exports.LOG_LEVEL, true),
        'Cleaner': GetCategory('Cleaner', exports.LOG_LEVEL, true),
        'Error': GetCategory('Error', exports.LOG_LEVEL, true),
        'IO': GetCategory('IO', exports.LOG_LEVEL, true),
        'ServerScripts': GetCategory('ServerScripts', exports.LOG_LEVEL, true),
        'Sql': GetCategory('Sql', exports.LOG_LEVEL, true),
        'PublicScripts': GetCategory('PublicScripts', exports.LOG_LEVEL, true),
        'Redis': GetCategory('Redis', exports.LOG_LEVEL, true),
        'Task': GetCategory('Task', exports.LOG_LEVEL, true),
    },
    pm2: true,
    disableClustering: !exports.ENABLE_LOG_PM2_INTERCOM,
};
