"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdvancedAIServiceImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const integration_config_1 = require("../config/integration.config");
const billing_batch_model_1 = require("../models/billing-batch.model");
const promises_shared_1 = require("../shared/promises.shared");
const request_shared_1 = require("../shared/request.shared");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'ADVANCED_AI_SERVICE';
const CURRENT_IMPLEMENTATION = 'AdvancedAIServiceImplementation';
class AdvancedAIServiceImplementation {
    constructor(vcc) {
        this.vcc = null;
        this.urlBase = `${integration_config_1.ADVANCED_AI_API_PROTOCOL}://${integration_config_1.ADVANCED_AI_API_URL}`;
        this.apikey = null;
        this.retry = 0;
        this.maxRetries = 2;
        this.errorData = new billing_batch_model_1.BatchError();
        this.vcc = vcc;
    }
    Query(appId, intents, text) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.Query.name}] [vcc::appId => ${this.vcc}::${appId}]: `;
        let postOptions = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetAdvancedAIEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let prompt = 'Eres un asistente que debe evaluar el texto ingresado por el usuario, estos son los criterios (Los nombres de criterio están con **nombre de criterio**): ';
            intents.forEach((intent, index) => {
                prompt += `${index + 1}. **${intent.id}**: "${intent.prompt}" `;
                if (intent.examples?.length) {
                    prompt += 'Ejemplos: ';
                    intent.examples.forEach(e => {
                        prompt += `- ${e.text} `;
                    });
                }
            });
            prompt += `Para cada criterio debes evaluar el cumplimiento (1 para sí y 0 para no) y responder sin falta siempre con un objeto json en este formato [{"intent": "nombre de criterio1", "score": cumplimiento}, {"intent": "nombre de criterio2", "score": cumplimiento}....] - **Nota**: Además me debes especificar la razón (con el texto específico obligatoriamente si hubiese y no debe tener ninguna comilla) de por qué marcaste 0 o 1.`;
            postOptions = {
                uri: `${this.urlBase}/genai/api/utilities/chat-completion`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                body: {
                    model: 'gpt-4o-mini-2024-07-18',
                    messages: [
                        {
                            "author": "system",
                            "text": prompt
                        },
                        {
                            "author": "agent",
                            "text": text
                        }
                    ],
                    temperature: 0,
                    tool_choice: {
                        type: "function",
                        function: {
                            name: "execute_query"
                        }
                    },
                    tools: [
                        {
                            type: "function",
                            function: {
                                name: "execute_query",
                                description: "Responde la pregunta del usuario",
                                parameters: {
                                    type: "object",
                                    properties: {
                                        result: {
                                            type: "string",
                                            description: "Respuesta para el usuario"
                                        }
                                    },
                                    required: [
                                        "result"
                                    ]
                                }
                            }
                        }
                    ]
                },
                retryProtocol: true
            };
            return request_shared_1.SimplePostRequest(postOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_QUERY_FAILED';
            if (!result?.message?.function_calls?.length)
                throw 'SERVER_ERROR_NO_RESULTS_FOR_REQUEST';
            let parsedResponse = null;
            try {
                parsedResponse = JSON.parse(result.message.function_calls[0].content);
                if (!parsedResponse.result.endsWith('"}]'))
                    parsedResponse.result.replace('}]', '"}]');
                if (!parsedResponse.result.endsWith('"}]}'))
                    parsedResponse.result.replace('}]}', '"}]');
                parsedResponse = JSON.parse(parsedResponse.result);
            }
            catch (e) {
                logger.error(`${logLabel} Catch: `, e);
                logger.error(`${logLabel} Failed response: `, result);
                throw 'SERVER_ERROR_ADVANCED_AI_QUERY_PARSE_FAILED';
            }
            let resultIntents = [];
            parsedResponse.map(i => {
                resultIntents.push({
                    intent: intents.find(intent => intent.id === i.intent).name,
                    score: i.score,
                    reason: i.reason ? i.reason : null
                });
            });
            let parseResult = {
                intents: resultIntents,
                usage: result.usage
            };
            logger.info(`${logLabel} FormattedResult: ` + JSON.stringify(parseResult));
            return Promise.resolve(parseResult);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            logger.error(`${logLabel} Error's body:`, postOptions);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_ADVANCED_AI_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetAdvancedAIEngineToken() {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetAdvancedAIEngineToken] [vcc => ${this.vcc}]: `;
        let cacheUpdateNeeded = false;
        return redis_service_1.Redis.GetAdvancedAIToken(this.vcc)
            .then(result => {
            if (result) {
                this.apikey = result;
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetAdvancedAIToken(this.vcc);
            }
        })
            .then(result => {
            if (result) {
                this.apikey ?? (this.apikey = result.token);
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveAdvancedAIToken(this.vcc, this.apikey);
                return Promise.resolve(true);
            }
            else
                throw 'SERVER_ERROR_ADVANCED_AI_TOKEN_NOT_FOUND';
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            return Promise.reject(errorMsg);
        });
    }
    ErroHandler(err, callback) {
        this.retry += 1;
        if (this.retry <= this.maxRetries) {
            console.log(`[${CURRENT_IMPLEMENTATION}::ErroHandler] Retry ${this.retry}/${this.maxRetries}...`);
            return new Promise((resolve, reject) => {
                setTimeout(() => {
                    return resolve(callback);
                }, 10000);
            });
        }
        else {
            console.log(`[${CURRENT_IMPLEMENTATION}::ErroHandler] Retry ${this.retry}`, err);
            return Promise.reject(err);
        }
    }
}
exports.AdvancedAIServiceImplementation = AdvancedAIServiceImplementation;
