"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StartProcess = exports.EvaluatePerformance = exports.EvaluateReloadChildTask = exports.SleepPromise = exports.GetEnvironment = exports.GetBlocksContentById = exports.GetTask = void 0;
const log4js = require("log4js");
const os = require("os");
const redis_service_1 = require("../db/redis.service");
const mssql_service_1 = require("../db/mssql.service");
const server_shared_1 = require("../shared/server.shared");
const background_config_1 = require("../config/background.config");
let logger = log4js.getLogger("Background");
function GetTask(taskId) {
    let cacheUpdateNeeded = false;
    let referenceTask;
    let referenceRedisTask;
    return redis_service_1.Redis.GetTask(taskId)
        .then((resultTask) => {
        referenceRedisTask = resultTask;
        if (referenceRedisTask) {
            return Promise.resolve(resultTask);
        }
        else {
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetTask(taskId);
        }
    }).then((resultTask) => {
        referenceTask = resultTask;
        if (referenceTask && !referenceRedisTask && cacheUpdateNeeded) {
            return redis_service_1.Redis.SaveTask(taskId, referenceTask);
        }
        else {
            return Promise.resolve(null);
        }
    }).then((result) => {
        if (referenceTask)
            return Promise.resolve(referenceTask);
        return Promise.resolve(null);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        return Promise.resolve(null);
    });
}
exports.GetTask = GetTask;
function GetBlocksContentById(instance) {
    let cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetBlocksContentById(instance)
        .then((redisBlocks) => {
        if (redisBlocks) {
            return Promise.resolve(redisBlocks);
        }
        else {
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetBlocks(instance, null, null, null, null, null, null);
        }
    }).then((blocks) => {
        if (blocks) {
            if (cacheUpdateNeeded) {
                redis_service_1.Redis.SaveBlocks(instance, blocks);
                let blocksById = {};
                blocks.forEach(block => {
                    if (block && block.id)
                        blocksById[block.id] = block.content;
                });
                return Promise.resolve(blocksById);
            }
            else {
                return Promise.resolve(blocks);
            }
        }
        else {
            logger.info('Se consulto dato de GetBlocks que no se encuentra definido en base de datos.');
            return Promise.resolve(null);
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.info('Error al ejecutar GetBlocksContentById: ' + errMsg);
    });
}
exports.GetBlocksContentById = GetBlocksContentById;
function GetEnvironment(instance, id) {
    let cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetEnvironment(instance, id)
        .then((redisEnvironment) => {
        if (redisEnvironment) {
            return Promise.resolve(redisEnvironment);
        }
        else {
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetEnvironment(instance, id);
        }
    }).then((environment) => {
        if (environment) {
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SaveEnvironment(instance, environment);
            return Promise.resolve(environment);
        }
        else {
            logger.info('Se consulto dato de GetEnvironment para \'' + id + '\' que no se encuentra definido en base de datos.');
            return Promise.resolve(null);
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.info('Error al ejecutar GetEnvironment: ' + errMsg);
    });
}
exports.GetEnvironment = GetEnvironment;
function SleepPromise(secondTime) {
    return new Promise(function (resolve, reject) {
        setTimeout(resolve, (1000 * secondTime));
    });
}
exports.SleepPromise = SleepPromise;
function EvaluateReloadChildTask(queue, items, timeRangeToEvaluateInSeconds, evaluateTimeInSeconds, waitingTimeToStopTaskInSeconds) {
    let requiresEvaluation = false;
    let requiresReload = false;
    let now;
    let updateDate;
    let savedItems = 0;
    return Promise.resolve(true)
        .then(result => {
        return redis_service_1.Redis.GetTaskBackgroundItems(queue);
    }).then(result => {
        savedItems = result || 0;
        now = Date.now();
        return redis_service_1.Redis.GetTaskBackgroundStatus(queue);
    }).then(result => {
        updateDate = result;
        if (savedItems > 0 && items > 0 && (savedItems === items || updateDate + timeRangeToEvaluateInSeconds * 1000 < now)) {
            logger.info('EvaluateReloadChildTask misma cantidad de items', savedItems);
            requiresEvaluation = true;
            return SleepPromise(evaluateTimeInSeconds);
        }
        return redis_service_1.Redis.SaveTaskBackgroundItems(queue, (items || 0));
    }).then(result => {
        if (requiresEvaluation) {
            now = Date.now();
            return redis_service_1.Redis.GetTaskBackgroundStatus(queue);
        }
        return null;
    }).then(result => {
        updateDate = result || now;
        if (requiresEvaluation && updateDate + timeRangeToEvaluateInSeconds * 1000 < now) {
            logger.info('EvaluateReloadChildTask marcar para reload', savedItems, updateDate);
            requiresReload = true;
            return redis_service_1.Redis.SaveTaskBackgroundStop(queue, true);
        }
        return true;
    }).then(result => {
        if (requiresReload)
            return SleepPromise(waitingTimeToStopTaskInSeconds);
        return true;
    }).then(result => {
        if (requiresReload)
            return redis_service_1.Redis.DeleteTaskBackgroundItems(queue);
        return true;
    }).then(result => {
        if (requiresReload)
            return redis_service_1.Redis.DeleteTaskBackgroundStatus(queue);
        return true;
    }).then(result => {
        if (requiresReload)
            return redis_service_1.Redis.SaveTaskBackgroundStop(queue, false);
        return true;
    }).then(result => {
        if (requiresReload)
            return Promise.resolve(true);
        return Promise.resolve(false);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.info('EvaluateReloadChildTask', errMsg);
        return Promise.resolve(false);
    });
}
exports.EvaluateReloadChildTask = EvaluateReloadChildTask;
function EvaluatePerformance() {
    return Promise.resolve(true)
        .then(result => {
        let loadAvgThreshold = 120;
        let memPercThreshold = 90;
        let cpuCount = os.cpus().length;
        let loadAvg = os.loadavg()[0];
        let loadAvgPerc = 100.00 * loadAvg / cpuCount;
        let memUsagePerc = 100.00 * (1.00 - server_shared_1.GetAvailableMemory() / os.totalmem());
        logger.info('GetPerformance: cpuCount', cpuCount, 'loadAvg', loadAvg, 'loadAvgPerc', loadAvgPerc, 'memUsagePerc', memUsagePerc);
        if (loadAvgPerc > loadAvgThreshold || memUsagePerc > memPercThreshold) {
            let delay = (Math.floor(Math.random() * 3) || 3) * 100;
            logger.info(`GetPerformance: Los recursos del servidor son demasiado bajos se espera un tiempo(${delay}s) antes de empezar(load avg: ${loadAvg.toFixed(2)} / mem usage: ${memUsagePerc.toFixed(2)}%)`);
            return SleepPromise(delay);
        }
        return Promise.resolve(true);
    }).catch(err => {
        logger.info('GetPerformance:', err);
        return Promise.resolve(false);
    });
}
exports.EvaluatePerformance = EvaluatePerformance;
function StartProcess(stopQueue, isSupervisor, evaluatePerformace, propagateError) {
    let requiredStop = false;
    return Promise.resolve(true)
        .then(_ => {
        return redis_service_1.Redis.GetTaskBackgroundStop(stopQueue);
    })
        .then((result) => {
        requiredStop = result;
        if (result && propagateError && !isSupervisor)
            throw 'RELOAD|' + stopQueue;
        if (result && isSupervisor)
            return SleepPromise(background_config_1.TIME_TO_STOP_THE_SUPERVISOR_IN_SECONDS);
        return Promise.resolve(true);
    })
        .then(_ => {
        if (requiredStop && propagateError && isSupervisor)
            throw 'RELOAD|' + stopQueue;
        if (evaluatePerformace)
            return EvaluatePerformance();
        return Promise.resolve(true);
    })
        .catch(err => {
        if (propagateError)
            return Promise.reject(err);
        return Promise.resolve(false);
    });
}
exports.StartProcess = StartProcess;
